@@ -1,11 +1,11 @@
 use std::{env, io};
 
-use bytes::BytesMut;
 use futures::StreamExt;
 use log::info;
 use ntex::http::header::HeaderValue;
 use ntex::http::{HttpService, Request, Response};
 use ntex::server::Server;
+use ntex::util::BytesMut;
 
 async fn handle_request(mut req: Request) -> Result<Response, io::Error> {
     let mut body = BytesMut::new();
