@@ -1,13 +1,13 @@
 //! Framed transport dispatcher
-use std::task::{Context, Poll};
 use std::{
-    cell::Cell, cell::RefCell, future::Future, pin::Pin, rc::Rc, time::Duration,
-    time::Instant,
+    cell::Cell, cell::RefCell, future::Future, pin::Pin, rc::Rc, task::Context,
+    task::Poll, time, time::Instant,
 };
 
 use crate::codec::{AsyncRead, AsyncWrite, Decoder, Encoder};
 use crate::framed::{DispatchItem, Read, ReadTask, State, Timer, Write, WriteTask};
 use crate::service::{IntoService, Service};
+use crate::time::Seconds;
 use crate::util::Either;
 
 type Response<U> = <U as Encoder>::Item;
@@ -39,7 +39,7 @@ where
     st: Cell<DispatcherState>,
     state: State,
     timer: Timer,
-    ka_timeout: u16,
+    ka_timeout: Seconds,
     ka_updated: Cell<Instant>,
     error: Cell<Option<S::Error>>,
     shared: Rc<DispatcherShared<S, U>>,
@@ -118,10 +118,10 @@ where
         timer: Timer,
     ) -> Self {
         let updated = timer.now();
-        let ka_timeout: u16 = 30;
+        let ka_timeout = Seconds(30);
 
         // register keepalive timer
-        let expire = updated + Duration::from_secs(ka_timeout as u64);
+        let expire = updated + time::Duration::from(ka_timeout);
         timer.register(expire, expire, &state);
 
         Dispatcher {
@@ -143,19 +143,18 @@ where
         }
     }
 
-    /// Set keep-alive timeout in seconds.
+    /// Set keep-alive timeout.
     ///
     /// To disable timeout set value to 0.
     ///
     /// By default keep-alive timeout is set to 30 seconds.
-    pub fn keepalive_timeout(mut self, timeout: u16) -> Self {
+    pub fn keepalive_timeout(mut self, timeout: Seconds) -> Self {
         // register keepalive timer
-        let prev = self.inner.ka_updated.get() + self.inner.ka();
-        if timeout == 0 {
+        let prev = self.inner.ka_updated.get() + time::Duration::from(self.inner.ka());
+        if timeout.is_zero() {
             self.inner.timer.unregister(prev, &self.inner.state);
         } else {
-            let expire =
-                self.inner.ka_updated.get() + Duration::from_secs(timeout as u64);
+            let expire = self.inner.ka_updated.get() + time::Duration::from(timeout);
             self.inner.timer.register(expire, prev, &self.inner.state);
         }
         self.inner.ka_timeout = timeout;
@@ -171,7 +170,7 @@ where
     /// To disable timeout set value to 0.
     ///
     /// By default disconnect timeout is set to 1 seconds.
-    pub fn disconnect_timeout(self, val: u16) -> Self {
+    pub fn disconnect_timeout(self, val: Seconds) -> Self {
         self.inner.state.set_disconnect_timeout(val);
         self
     }
@@ -226,7 +225,7 @@ where
         loop {
             match slf.st.get() {
                 DispatcherState::Processing => {
-                    let result = match slf.poll_service(&this.service, cx, read) {
+                    let result = match slf.poll_service(this.service, cx, read) {
                         Poll::Pending => return Poll::Pending,
                         Poll::Ready(result) => result,
                     };
@@ -285,7 +284,7 @@ where
                 }
                 // handle write back-pressure
                 DispatcherState::Backpressure => {
-                    let result = match slf.poll_service(&this.service, cx, read) {
+                    let result = match slf.poll_service(this.service, cx, read) {
                         Poll::Ready(result) => result,
                         Poll::Pending => return Poll::Pending,
                     };
@@ -459,12 +458,12 @@ where
         }
     }
 
-    fn ka(&self) -> Duration {
-        Duration::from_secs(self.ka_timeout as u64)
+    fn ka(&self) -> Seconds {
+        self.ka_timeout
     }
 
     fn ka_enabled(&self) -> bool {
-        self.ka_timeout > 0
+        self.ka_timeout.non_zero()
     }
 
     /// check keepalive timeout
@@ -484,7 +483,7 @@ where
         if self.ka_enabled() {
             let updated = self.timer.now();
             if updated != self.ka_updated.get() {
-                let ka = self.ka();
+                let ka = time::Duration::from(self.ka());
                 self.timer.register(
                     updated + ka,
                     self.ka_updated.get() + ka,
@@ -498,21 +497,24 @@ where
     /// unregister keep-alive timer
     fn unregister_keepalive(&self) {
         if self.ka_enabled() {
-            self.timer
-                .unregister(self.ka_updated.get() + self.ka(), &self.state);
+            self.timer.unregister(
+                self.ka_updated.get() + time::Duration::from(self.ka()),
+                &self.state,
+            );
         }
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use bytes::Bytes;
     use rand::Rng;
     use std::sync::{atomic::AtomicBool, atomic::Ordering::Relaxed, Arc, Mutex};
+    use std::time::Duration;
 
     use crate::codec::BytesCodec;
-    use crate::rt::time::sleep;
     use crate::testing::Io;
+    use crate::time::{sleep, Millis};
+    use crate::util::Bytes;
 
     use super::*;
 
@@ -534,7 +536,7 @@ mod tests {
             T: AsyncRead + AsyncWrite + Unpin + 'static,
         {
             let timer = Timer::default();
-            let ka_timeout = 1;
+            let ka_timeout = Seconds(1);
             let ka_updated = timer.now();
             let state = State::new();
             let io = Rc::new(RefCell::new(io));
@@ -578,8 +580,8 @@ mod tests {
         let (disp, _) = Dispatcher::debug(
             server,
             BytesCodec,
-            crate::fn_service(|msg: DispatchItem<BytesCodec>| async move {
-                sleep(Duration::from_millis(50)).await;
+            crate::service::fn_service(|msg: DispatchItem<BytesCodec>| async move {
+                sleep(Millis(50)).await;
                 if let DispatchItem::Item(msg) = msg {
                     Ok::<_, ()>(Some(msg.freeze()))
                 } else {
@@ -607,7 +609,7 @@ mod tests {
         let (disp, st) = Dispatcher::debug(
             server,
             BytesCodec,
-            crate::fn_service(|msg: DispatchItem<BytesCodec>| async move {
+            crate::service::fn_service(|msg: DispatchItem<BytesCodec>| async move {
                 if let DispatchItem::Item(msg) = msg {
                     Ok::<_, ()>(Some(msg.freeze()))
                 } else {
@@ -616,7 +618,7 @@ mod tests {
             }),
         );
         crate::rt::spawn(async move {
-            let _ = disp.disconnect_timeout(25).await;
+            let _ = disp.disconnect_timeout(Seconds(1)).await;
         });
 
         let buf = client.read().await.unwrap();
@@ -630,7 +632,7 @@ mod tests {
         assert_eq!(buf, Bytes::from_static(b"test"));
 
         st.close();
-        sleep(Duration::from_millis(200)).await;
+        sleep(Millis(1100)).await;
         assert!(client.is_server_dropped());
     }
 
@@ -643,7 +645,7 @@ mod tests {
         let (disp, state) = Dispatcher::debug(
             server,
             BytesCodec,
-            crate::fn_service(|_: DispatchItem<BytesCodec>| async move {
+            crate::service::fn_service(|_: DispatchItem<BytesCodec>| async move {
                 Err::<Option<Bytes>, _>(())
             }),
         );
@@ -684,7 +686,7 @@ mod tests {
         let (disp, state) = Dispatcher::debug(
             server,
             BytesCodec,
-            crate::fn_service(move |msg: DispatchItem<BytesCodec>| {
+            crate::service::fn_service(move |msg: DispatchItem<BytesCodec>| {
                 let data = data2.clone();
                 async move {
                     match msg {
@@ -717,7 +719,7 @@ mod tests {
         let buf = client.read_any();
         assert_eq!(buf, Bytes::from_static(b""));
         client.write("GET /test HTTP/1\r\n\r\n");
-        sleep(Duration::from_millis(25)).await;
+        sleep(Millis(25)).await;
 
         // buf must be consumed
         assert_eq!(client.remote_buffer(|buf| buf.len()), 0);
@@ -727,11 +729,11 @@ mod tests {
         assert_eq!(state.write().with_buf(|buf| buf.len()), 65536);
 
         client.remote_buffer_cap(10240);
-        sleep(Duration::from_millis(50)).await;
+        sleep(Millis(50)).await;
         assert_eq!(state.write().with_buf(|buf| buf.len()), 55296);
 
         client.remote_buffer_cap(45056);
-        sleep(Duration::from_millis(50)).await;
+        sleep(Millis(50)).await;
         assert_eq!(state.write().with_buf(|buf| buf.len()), 10240);
 
         // backpressure disabled
@@ -752,7 +754,7 @@ mod tests {
         let (disp, state) = Dispatcher::debug(
             server,
             BytesCodec,
-            crate::fn_service(move |msg: DispatchItem<BytesCodec>| {
+            crate::service::fn_service(move |msg: DispatchItem<BytesCodec>| {
                 let data = data2.clone();
                 async move {
                     match msg {
@@ -770,14 +772,17 @@ mod tests {
             }),
         );
         crate::rt::spawn(async move {
-            let _ = disp.keepalive_timeout(0).keepalive_timeout(1).await;
+            let _ = disp
+                .keepalive_timeout(Seconds::ZERO)
+                .keepalive_timeout(Seconds(1))
+                .await;
         });
 
-        state.set_disconnect_timeout(1);
+        state.set_disconnect_timeout(Seconds(1));
 
         let buf = client.read().await.unwrap();
         assert_eq!(buf, Bytes::from_static(b"GET /test HTTP/1\r\n\r\n"));
-        sleep(Duration::from_millis(3100)).await;
+        sleep(Millis(3500)).await;
 
         // write side must be closed, dispatcher should fail with keep-alive
         let flags = state.flags();
@@ -800,10 +805,10 @@ mod tests {
         let (disp, _) = Dispatcher::debug(
             server,
             BytesCodec,
-            crate::fn_service(move |msg: DispatchItem<BytesCodec>| {
+            crate::service::fn_service(move |msg: DispatchItem<BytesCodec>| {
                 handled2.store(true, Relaxed);
                 async move {
-                    sleep(Duration::from_millis(50)).await;
+                    sleep(Millis(50)).await;
                     if let DispatchItem::Item(msg) = msg {
                         Ok::<_, ()>(Some(msg.freeze()))
                     } else {
@@ -816,7 +821,7 @@ mod tests {
         crate::rt::spawn(async move {
             let _ = disp.await;
         });
-        sleep(Duration::from_millis(50)).await;
+        sleep(Millis(50)).await;
 
         assert!(handled.load(Relaxed));
     }
