@@ -1,13 +1,13 @@
-use std::task::{Context, Poll};
-use std::{future::Future, io, mem, net, pin::Pin, time::Duration};
+use std::{future::Future, io, mem, net, pin::Pin, task::Context, task::Poll};
 
+use async_channel::{unbounded, Receiver};
+use async_oneshot as oneshot;
+use futures_core::Stream;
 use log::{error, info};
 use socket2::{Domain, SockAddr, Socket, Type};
-use tokio::sync::mpsc::{unbounded_channel, UnboundedReceiver};
-use tokio::sync::oneshot;
 
-use crate::rt::{net::TcpStream, spawn, time::sleep, System};
-use crate::util::join_all;
+use crate::rt::{net::TcpStream, spawn, System};
+use crate::{time::sleep, time::Millis, util::join_all};
 
 use super::accept::{AcceptLoop, AcceptNotify, Command};
 use super::config::{ConfiguredService, ServiceConfig};
@@ -17,7 +17,7 @@ use super::socket::Listener;
 use super::worker::{self, Worker, WorkerAvailability, WorkerClient};
 use super::{Server, ServerCommand, ServerStatus, Token};
 
-const STOP_DELAY: Duration = Duration::from_millis(300);
+const STOP_DELAY: Millis = Millis(300);
 
 /// Server builder
 pub struct ServerBuilder {
@@ -29,9 +29,9 @@ pub struct ServerBuilder {
     sockets: Vec<(Token, String, Listener)>,
     accept: AcceptLoop,
     exit: bool,
-    shutdown_timeout: Duration,
+    shutdown_timeout: Millis,
     no_signals: bool,
-    cmd: UnboundedReceiver<ServerCommand>,
+    cmd: Receiver<ServerCommand>,
     server: Server,
     notify: Vec<oneshot::Sender<()>>,
 }
@@ -45,7 +45,7 @@ impl Default for ServerBuilder {
 impl ServerBuilder {
     /// Create new Server builder instance
     pub fn new() -> ServerBuilder {
-        let (tx, rx) = unbounded_channel();
+        let (tx, rx) = unbounded();
         let server = Server::new(tx);
 
         ServerBuilder {
@@ -57,7 +57,7 @@ impl ServerBuilder {
             accept: AcceptLoop::new(server.clone()),
             backlog: 2048,
             exit: false,
-            shutdown_timeout: Duration::from_secs(30),
+            shutdown_timeout: Millis::from_secs(30),
             no_signals: false,
             cmd: rx,
             notify: Vec::new(),
@@ -116,15 +116,15 @@ impl ServerBuilder {
         self
     }
 
-    /// Timeout for graceful workers shutdown in seconds.
+    /// Timeout for graceful workers shutdown.
     ///
     /// After receiving a stop signal, workers have this much time to finish
     /// serving requests. Workers still alive after the timeout are force
     /// dropped.
     ///
     /// By default shutdown timeout sets to 30 seconds.
-    pub fn shutdown_timeout(mut self, sec: u64) -> Self {
-        self.shutdown_timeout = Duration::from_secs(sec);
+    pub fn shutdown_timeout<T: Into<Millis>>(mut self, timeout: T) -> Self {
+        self.shutdown_timeout = timeout.into();
         self
     }
 
@@ -323,11 +323,11 @@ impl ServerBuilder {
 
     fn handle_cmd(&mut self, item: ServerCommand) {
         match item {
-            ServerCommand::Pause(tx) => {
+            ServerCommand::Pause(mut tx) => {
                 self.accept.send(Command::Pause);
                 let _ = tx.send(());
             }
-            ServerCommand::Resume(tx) => {
+            ServerCommand::Resume(mut tx) => {
                 self.accept.send(Command::Resume);
                 let _ = tx.send(());
             }
@@ -386,10 +386,10 @@ impl ServerBuilder {
                     spawn(async move {
                         let _ = join_all(futs).await;
 
-                        if let Some(tx) = completion {
+                        if let Some(mut tx) = completion {
                             let _ = tx.send(());
                         }
-                        for tx in notify {
+                        for mut tx in notify {
                             let _ = tx.send(());
                         }
                         if exit {
@@ -405,10 +405,10 @@ impl ServerBuilder {
                             System::current().stop();
                         });
                     }
-                    if let Some(tx) = completion {
+                    if let Some(mut tx) = completion {
                         let _ = tx.send(());
                     }
-                    for tx in notify {
+                    for mut tx in notify {
                         let _ = tx.send(());
                     }
                 }
@@ -451,7 +451,7 @@ impl Future for ServerBuilder {
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         loop {
-            match Pin::new(&mut self.cmd).poll_recv(cx) {
+            match Pin::new(&mut self.cmd).poll_next(cx) {
                 Poll::Ready(Some(it)) => self.as_mut().get_mut().handle_cmd(it),
                 Poll::Ready(None) => return Poll::Pending,
                 Poll::Pending => return Poll::Pending,
@@ -520,8 +520,7 @@ mod tests {
     #[cfg(unix)]
     #[crate::rt_test]
     async fn test_signals() {
-        use std::sync::mpsc;
-        use std::{net, thread, time};
+        use std::{net, sync::mpsc, thread};
 
         fn start(tx: mpsc::Sender<(Server, net::SocketAddr)>) -> thread::JoinHandle<()> {
             thread::spawn(move || {
@@ -551,11 +550,11 @@ mod tests {
             let h = start(tx);
             let (srv, addr) = rx.recv().unwrap();
 
-            crate::rt::time::sleep(time::Duration::from_millis(300)).await;
+            crate::time::sleep(Millis(300)).await;
             assert!(net::TcpStream::connect(addr).is_ok());
 
             srv.signal(*sig);
-            crate::rt::time::sleep(time::Duration::from_millis(300)).await;
+            crate::time::sleep(Millis(300)).await;
             assert!(net::TcpStream::connect(addr).is_err());
             let _ = h.join();
         }
